/*
Copyright (C) 2009 -  Claudio Salvadori & Paolo Pagano 
This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/

/**
 *\file documentation.h
 *\brief  Documentation pages
 */

/** 
* \mainpage DAQ: a ROOT based Data AcQuisition platform for debugging embedded devices
* \section motivation Motivation
* In Embedded Systems the validation of software solutions
* is quite difficult with respect to high-end devices 
* for the lack of ordinary debugging tools like display,
* file system, etc.
* The communication towards a PC permits to debug the software code making
* use of tools available in high-end devices only (awk, grep, find, etc.)
* Moreover it is quite beneficial to implement data
* serialization in order to assess the performances of the embedded system
* firmware (implementing some kind of logic)
* with respect to full-fledged algorithms available in high end devices only.
* We propose this framework including serialization and post-processing virtues
* coming from the ROOT framework developed at CERN with standardization and
* extendibility feautures coming from the xml language.
* We propose an example to test the performances 
* of a full-customed firmware for onboard image processing.
*
* \section arch Software architecture 
* The architecture of DAQ is based essentially on 4 blocks: 
* - \b xml \b parser to interpret the communication specs (physical specs, handshake feature, data pattern...);
* - \b data \b reader to read raw data directly from the API functions;
* - \b data \b formatter to format data following the given data pattern;
* - \b ROOT \b serializer to store data into a repository (ROOT file) on the filesystem.
*
* The working principle is based on the feature extraction from xml file and the creation of 2 data structures:
* one to define the reader tasks and one to describe the packet content in terms of fields (vector of metadata). For such puroposes the structs SDef and SMData are defined.
So the logic follows these steps:
* - start of the communication (initialization and handshake)
* - syncronization with the stream (by lenght or by the start and end symbols)
* - raw data reading
* - raw data processing (decoding and fragmentation in messages prior to interpretation)
* - data formatting following the metadata definition
*
* The output of this system is a vector of vector of \b SData, a structure
* that contains data formatted plus the information needed for the case of
* casting and processing.  
* \image latex general.eps "DAQ architecture"width=10cm 
* \image html general.png "DAQ architecture" width=2cm 
* \image latex Reader.eps "Reader structure" width=10cm 
* \image html Reader.png "Reader structure" width=2cm 
* The formatted data can be stored in a structured
* repository called ROOT designed by CERN to store data from  experiments
* in High Energy Physics. ROOT Framework implements procedures for data debugging
* permitting post-processing (off-line) of "structured" data: this
capability is very valuable whenever direct means for system evaluation
are absent asin the case of very simple embedded systems (having no display, no filesystem
reduced I/O functionalities).
*  Evidently to render flexible and adaptive these capabilities
* some coding intervention is necessary especially for the interface between DAQ
* and ROOT (files DAQSData.cxx and DAQPayload.h in the directory rootbulk).   
* \image html interface.png "Interface DAQ-ROOT" width=2cm
* \image latex interface.eps "Interface DAQ-ROOT" width=10cm
* \page usage Usage 
* \section installation Installation 
* DAQ platform is possible to compile under Linux and under Windows POSIX emulator Cygwin. To compile
* correctly DAQ platform is necessary to install:
* - Xerces lib (install directly from packet manager: libxerces-c2-dev and libxerces-c28)
* - FLTK libraries (install directly from packet manager: libfltk1.1 and libfltk1.1-dev). If is used Cygwin see also \ref cygwin "Cygwin issues"
* - JPEG libraries (install directly from packet manager: libjeg62 and libjeg62-dev)
* - ROOT framework (downlodable from http://root.cern.ch/drupal/). If is used Cygwin see also \ref cygwin "Cygwin issues".
*
**\subsection cygwin Cygwin issues 
* To use FLTK libraries on Cygwin it is needed to fix the jmorecfg.h file, replacing:
* - \verbatim #ifndef HAVE_BOOLEAN \endverbatim with \verbatim #if (!defined(HAVE_BOOLEAN)) && (!defined(_RPCNDR_H)) \endverbatim
* - \verbatim #ifndef XMD_H \endverbatim with \verbatim#if (!defined(XMD_H) && !defined(_BASETSD_H)) \endverbatim
*
* The only ROOT version working on Cygwin is the 5.11 version, that's included in this package. 
* \warning Remember that ROOT under Cygwin is unsupported.
*
* We provide the DAQ system for Cygwin as a "unsupported" product;
* We will not guarantee to help you in solving Cygwin/ Windows related bugs/ misbehaviors.
* \section conf System personalization
* \subsection impl How to add a module?
* In this subsection it is explained how to add a new stream class to interface to another communication device. To do this follow
* these steps:
* - Implement the class related to the communication device inheriting from IStreamDaq class.
* - Update the \ref schema with the name of the new communication device.
* - Update the enumeration of the devices in daq_enum.h  
* - Update the xml parsing procedure
* - Update the error enumeration
* - Update the CReader class
*
* \subsection conf How to customize DAQ to acquire and store data?
* In this subsection it is explained how to interface DAQ platform with the embedded system. To do this follow
* these steps:
* - customize the xml file following xml-schema (see \ref schema)
* - customize DAQSData.cxx and DAQPayload.h files to interface DAQ with ROOT repository
*
* \page schema Xml-Schema
* \include input.xsd
*
* \page socket EXAMPLE : acquisition of a stream from a socket
* DAQ libraries give the possibilities to communicate and parse data from a socket, where the DAQ is the client
* and the source of the stream is the server. In this case it is implemented a simple server thread (see SocketExample.cpp)
* that after a simple handshake sends packets shaped like: <DAQ*NNNNNNNN>, where <, > are respectively the start and 
* the end syncronization character, and NNNNNNNN is an UINT32 counter.
* \section example_ How does SocketExample.cpp work?
* \dontinclude SocketExample.cpp
* This is the implementation of a socket server to communicate with DAQ; the port number is passed as an argument (i.e. 
* ./exe/SocketExample 2000). 
* \skip argv
* \until {
* Checking the correct sintax for SocketExample esecution: port number has to be passed as parameter
* \skip if
* \until }
* Opening a socket to be used in internet domain using TCP
* \skip sockfd
* \until }
* Filling the server address structure
* \skip bzero
* \until number
* Binding operation (server side only)
* \skip sockfd
* \until }
* Listening for clients connection (blocking function)
* \skip sockfd
* \until clilen
* Accepting a client and connection
* \skip newsockfd
* \until }
* Handshake:
* \{
* - Reading the handshaking word
* \skip bzero
* \until }
* - Checking if the handshaking packet is correct (it has to be 1) and answer...
* \skipline strcmp
* ... ACK if the packet is correct, ...
* \skip ACK
* \until }
* ...NAK otherwise
* \skip NAK
* \until }
* \}
* Infinite cycle for upgrading the counter and sending packets
* \skip while
* \until return
*
* \section xml_socket XML file example for socket acquisition
* This file.xml fixes the socket settings, the handshake word to initialize the communication, the packet lenght
* and the data topology in a packet.
* \dontinclude socketLinux.xml
* \subsection title Stream title (REQUIRED)
* \skipline title
* \subsection stream Stream definition (REQUIRED)
* \{
* - stream_type: type of stream (the allowed value are file, serial, pipe, socket). REQUIRED
* - stream_name: the name or the address of the file to be written (/dev/ttyUSB0 for serial, or IP-address or 'localhost' for socket, or the name of the file,
* or the pipe name. REQUIRED. 
* - stream_info: more info about the stream (i.e. the port number for the socket). REQUIRED
* In this case DAQ will use a socket that it has to connect to the IP-address 'localhost' through the port number 2000.
* \skipline stream
* - stream option (NOT REQUIRED): \n
* 1. decoder: used decoder type (allowable value are 'none' and 'xor') \n
* 2. esc_char: escape character (for decoder). Could be expressed like a character or like an hexadecimal number (see s_type_opt) \n
* 3. start_char: syncronization start char. Could be expressed like a character or like an hexadecimal number (see s_type_opt)  \n
* 4. end_char: syncronization end char. Could be expressed like a character or like an hexadecimal number (see s_type_opt)  \n
* 5. s_type_opt: the way to interpret the selected character: \n 
* ..a. 'char' all the character are interpreted like character; \n
* ..b. 'hex' all the character are interpreted like hexadecimal value.\n
* In this case the start character will be 0x02 and the end character will be 0x03.
* \skipline stream_opt
* \}
* \subsection handshake Handshake definition (REQUIRED)
* The attribute of the node handshake explain wich type of handshake is choosen:
* \{
* - 'none': no handshake
* - 'start': handshake is done to syncronize DAQ with the stream and/or to set-up the device.
* - 'standard': the packet to be parsed by DAQ is an answer to the unique handshake packet.
* \}
* Every handshake packet is describe by this field: (NOT REQUIRED)
* \{
* - handshake_send: packet to send (REQUIRED in case of 'start' and 'standard') 
* - handshake_ack: acknowledgement to the packet (only in case of 'start') 
* - handshake_nack: not acknowledgement to the packet (only in case of 'start')  
* - h_type_pkt: the way to interpret the selected character: \n 
* 1. 'char' all the character are interpreted like character; \n
* 2. 'hex' all the character are interpreted like hexadecimal value.\n
* \}
* In this case the handshake type is 'start', the packet to send is '1' (0x31), ACK packet is '1' (0x31) and NAK packet '0' (0x30).
* \skip handshake
* \until /handshake
* \subsection payload Payload definition  
* In this part is described how the packet to parse are organized:
* \{
* - length_pkt : total packet length in bytes (REQUIRED)
* - n data node as the number of the fields in the packet. Every node is composed with this attributes: \n
* ..1) data_name: name of the field (REQUIRED) \n
* ..2) data_position: absolute position from the start of the packet in number of bytes (REQUIRED)  \n
* ..3) data_size: size of the field in number of bytes (REQUIRED)  \n
* ..4) data_type: data type (INT8, UINT8, INT16, UINT16, INT32, UINT32, INT64, UINT64, FLOAT32, FLOAT64) (REQUIRED)  \n
* ..5) type: semantic of the data (DS_NOTHING, DS_TABLE, DS_IMAGE, DS_HISTOGRAM (REQUIRED) ) \n
* \}
* \skip packet_description
* \until /packet_description
* \warning These five character entities are assumed to be predeclared, and it is possible to use without declaring them:
* - "&lt;": the less-than character (<) starts element markup (the first character of a start-tag or an end-tag).
* - "&amp;": the ampersand character (&) starts entity markup (the first character of a character entity reference).
* - "&gt;": the greater-than character (>) ends a start-tag or an end-tag.
* - "&quot;": the double-quote character (") can be symbolised with this character entity reference when you need to 
* embed a double-quote inside a string which is already double-quoted.
* - "&apos;": the apostrophe or single-quote character (') can be symbolised with this character entity reference when 
* you need to embed a single-quote or apostrophe inside a string which is already single-quoted.
* \subsection xml socketLinux.xml
* \include socketLinux.xml
*
* \section pay_socket Payload analysis example
* In this case the DAQEvent class is projected to store data from the socket, and in the main(...) function
* (in DAQ.cpp) the event->Build(...) function is called every packet received.
* \verbatim
 		event->Build(ev++, data_formatted);
		tree->Fill();  //fill the tree
* \endverbatim
* \section post_socket Post-processing socket stream
* The file LoopDAQEvent.cpp is the application to process data stored inside ROOT repository ( default name is \b test.root ). The logic
* of this simple programm is:
* - read the element
* \verbatim
 	TClonesArray *myarray = event->GetDAQSDatas();
      	if (myarray) mydaq = (DAQSData *) myarray->First();
  \endverbatim
* - display data
* \verbatim
 	cout << "GetMessage: " << mydaq->getMessage() << endl; 
	cout << "GetCounter: " << mydaq->getCounter() << endl;
  \endverbatim
* - process the counter
* \verbatim
 	nselected++;
	count += (Double_t)mydaq->getCounter() / (nevent - nselected);
  \endverbatim
* \page serial EXAMPLE : serial acquisition of a picture from HV7131GP and FLEX
* The firmware is downlodable from http://www.evidence.eu.com/content/view/329/266/.
*
* \section xml_image XML file example for image acquisition via RS-232
* This file.xml fixes the RS-232 settings, the handshake word to initialize the camera, the packet lenght
* and the data topology in a packet.
* \dontinclude serialImageLinux.xml
* \subsection Title Stream title (REQUIRED)
* \skipline title
* \subsection Stream Stream definition (REQUIRED)
* \{
* - stream_type: type of stream (the allowed value are file, serial, pipe, socket). REQUIRED
* - stream_name: the name or the address of the file to be written (/dev/ttyUSB0 for serial, or IP-address or 'localhost' for socket, or the name of the file,
* or the pipe name. (REQUIRED)
* - stream_info: more info about the stream (i.e. the port number for the socket). REQUIRED
* \skipline stream
* - serial preset (REQUIRED only for SERIAL): \n
* 1. baud_rate: baud rate for serial communication (to check the allowable values see input.xsd schema file) \n
* 2. data_bits: length in bits of a character received (to check the allowable values see input.xsd schema file) \n
* 3. parity: parity bit information \n
* 4. stop_bits : number of stop bits \n
* The serial stream selected will have baud-rate 115200, 8 bits for character, no parity bit and 1 stop bit.
* \skip baud
* \until baud
* \}
* \subsection Handshake Handshake definition
* The attribute of the node handshake explain wich type of handshake is choosen:
* \{
* - 'none': no handshake
* - 'start': handshake is done to syncronize DAQ with the stream and/or to set-up the device.
* - 'standard': the packet to be parsed by DAQ is an answer to the unique handshake packet.
* \}
* Every handshake packet is describe by this field:
* \{
* - handshake_send: packet to send (REQUIRED in case of 'start' and 'standard') 
* - handshake_ack: acknowledgement to the packet (only in case of 'start') 
* - handshake_nack: not acknowledgement to the packet (only in case of 'start')  
* - h_type_pkt: the way to interpret the selected character: \n 
* 1. 'char' all the character are interpreted like character; \n
* 2. 'hex' all the character are interpreted like hexadecimal value.\n
* \}
* In this case the handshake type is 'start', the packets to send are:
* \{
* - '1' (0x31)
* - '*01*01*' (0x2A 0x30 0x31 0x2A 0x30 0x31 0x2A)
* \}
* ACK packets are:
* \{
* - '1' (0x31)
* - '1' (0x31)
* \}
and NAK packets are: 
* \{
* - '0' (0x30)
* - '0' (0x30)
* \}
* \skip handshake
* \until /handshake
*
* \subsection payload Payload definition (REQUIRED)
* In this part is described how the packet to parse are organized:
* - length_pkt : total packet length in bytes (REQUIRED)
* - n data node as the number of the fields in the packet. Every node is composed with this attributes: \n
* ..1) data_name: name of the field \n
* ..2) data_position: absolute position from the start of the packet in number of bytes \n
* ..3) data_size: size of the field in number of bytes \n
* ..4) data_type: data type (INT8, UINT8, INT16, UINT16, INT32, UINT32, INT64, UINT64, FLOAT32, FLOAT64) \n
* ..5) type: semantic of the data (DS_NOTHING, DS_TABLE, DS_IMAGE, DS_HISTOGRAM) \n
* \skip packet_description
* \until /packet_description
* \warning These five character entities are assumed to be predeclared, and it is possible to use without declaring them:
* - "&lt;": the less-than character (<) starts element markup (the first character of a start-tag or an end-tag).
* - "&amp;": the ampersand character (&) starts entity markup (the first character of a character entity reference).
* - "&gt;": the greater-than character (>) ends a start-tag or an end-tag.
* - "&quot;": the double-quote character (") can be symbolised with this character entity reference when you need to 
* embed a double-quote inside a string which is already double-quoted.
* - "&apos;": the apostrophe or single-quote character (') can be symbolised with this character entity reference when 
* you need to embed a single-quote or apostrophe inside a string which is already single-quoted.
* \subsection Xml serialImageLinux.xml
* \include serialImageLinux.xml
*
* \section pay_image Payload analysis example
* In this case the DAQEvent class is projected to store an image of 19200 bytes (160x120 pixel), and in the main(...) function
* (in DAQ.cpp) the event->Build(...) function is called every packet received.
* \verbatim
 		event->Build(ev++, data_formatted);
		tree->Fill();  //fill the tree
* \endverbatim
*
* \section post_image Post-processing images
* The file LoopDAQEvent.cpp is the application to process the images stored inside ROOT repository ( default name is \b test.root ). The paradigm
* of this simple programm is:
* - read the element
* \verbatim
 	TClonesArray *myarray = event->GetDAQSDatas();
      	if (myarray) mydaq = (DAQSData *) myarray->First();
  \endverbatim
* - calculate the likely-hood value using FDY and filling histogram
* \verbatim
 	my[graph_abscissa] = fdy_cd(mydaq->getSImage(), 19200);
       	TAxis *ax = hp[1]->GetXaxis();
      	hp[1]->SetBinContent((ax->GetBinCenter((Float_t)graph_abscissa)) + 2, y[graph_abscissa]);
      	graph_abscissa += 1;
  \endverbatim
* - show stored image
* \verbatim
 	mydaq->DAQSDataInit();
     	mydaq->DAQSDataRender();
  \endverbatim
*
*/
